###################################
#ESTABLECE EL DIRECTORIO DE TRABAJO
###################################

################################
#INSTALACION Y CARGA DE PAQUETES
################################
#NOTA: SOLO ES NECESARIO INSTALARLOS UNA VEZ. DESACTIVA ESTAS LÍNEAS PARA LA PRÓXIMA SESIÓN
#INSTALA PAQUETE DISMO Y TODAS SUS DEPENDENCIAS (EJECUTAR UNA SOLA VEZ)

install.packages("tree", dep=TRUE) #regression trees
install.packages("partykit", dep=TRUE) #conditional inference trees (requires rJava)
install.packages("randomForest", dep=TRUE) #random forest
install.packages("mgcv", dep=TRUE) #GAM
install.packages("plotmo", dep=TRUE) #response curves
install.packages("earth", dep=TRUE) #MARS
install.packages("maxnet", dep=TRUE) #MAXENT
install.packages("vegan", dep=TRUE) #NMDS
install.packages("lm.beta", dep=TRUE) #coeficientes estandarizados para GLM
install.packages("glmulti", dep=TRUE) #selección de variables para GLM
install.packages("gbm", dep=TRUE) #gradient boosting modelling (boosted regression trees)
install.packages("ggplot2", dep=TRUE) #para plotear datos


#CARGA LAS LIBRERIAS NECESARIAS (EJECUTAR SIEMPRE QUE TRABAJES CON EL SCRIPT)
library(dismo) #LIBRERIA PARA MODELOS DE DISTRIBUCION
library(plotmo) #LIBRERIA PARA VISUALIZACION DE CURVAS DE RESPUESTA
library(mgcv) #OTRA LIBRERIA PARA GAM
library(randomForest) #RANDOM FOREST
library(tree) #ÁRBOLES DE CLASIFICACION
library(partykit) #ÁRBOLES DE INFERENCIA CONDICIONAL
library(randomForest) #RANDOM FOREST
library(mgcv) #GAM
library(plotmo) #PLOTEAR MODELOS
library(earth) #MARS
library(maxnet) #MAXENT
library(dismo) #SDMs EN GENERAL
library(raster)
library(vegan) #PARA NON METRIC MULTIDIMENSIONAL SCALING
library(lm.beta) #COEFICIENTES ESTANDARIZADOS EN LM Y GLM
library(glmulti) #SELECCION DE MODELOS GLM
library(gbm) #BOOSTED REGRESSION TREES (GRADIENT BOOST MODELING)
library(devtools) #TO INSTALL 
library(ggplot2) #PLOTS
library(grid) #PLOTS
library(gridExtra) #PLOTS
library(RColorBrewer) #ESCALAS DE COLOR

#CARGA FUNCIONES
source("funcionesSDM_taller2.R")

#CARGA LAS PRESENCIAS Y LAS VARIABLES
unzip("./data/variables_y_presencia.zip", exdir="./data/")
load("./data/variables_y_presencia/variables_y_presencia.Rdata")

#CREA CARPETA PARA GUARDAR LOS MODELOS
dir.create("./resultados")

################################################
################################################
#VISUALIZANDO LOS DATOS
################################################
################################################

################################################
#PARA UNA SOLA VARIABLE AMBIENTAL
################################################

#guardamos todo en un solo pdf
pdf("./resultados/analisis_exploratorio1.pdf", width=15, height=6, pointsize=30)

#itera por las variables
for (variable in names(variables)){

par(mfrow=c(1,3))

#límites de la variable en los gráficos
variable.min<-min(presencia.background[, variable])
variable.max<-max(presencia.background[, variable])

#density plot
#------------
#presencias vs. background
d0<-density(presencia.background[presencia.background$presencia==0, variable], from=variable.min, to=variable.max)
d1<-density(presencia.background[presencia.background$presencia==1, variable], from=variable.min, to=variable.max)
plot(d0, col="red", xlim=c(variable.min,variable.max), lwd=3, main=paste(variable, " vs. background", sep=""), xlab=variable, ylim=c(0, max(d0$y, d1$y)))
lines(d1, col="forestgreen", lwd=3)

#presencias vs ausencias
d0<-density(presencia.ausencia[presencia.ausencia$presencia==0, variable], from=variable.min, to=variable.max)
d1<-density(presencia.ausencia[presencia.ausencia$presencia==1, variable], from=variable.min, to=variable.max)
plot(d0, col="red", xlim=c(variable.min,variable.max), lwd=3, main=paste(variable, " vs. ausencias", sep=""), xlab=variable, ylim=c(0, max(d0$y, d1$y)))
lines(d1, col="forestgreen", lwd=3)

#presencias vs pseudoausencias
d0<-density(presencia.pseudoausencia[presencia.pseudoausencia$presencia==0, variable], from=variable.min, to=variable.max)
d1<-density(presencia.pseudoausencia[presencia.pseudoausencia$presencia==1, variable], from=variable.min, to=variable.max)
plot(d0, col="red", xlim=c(variable.min,variable.max), lwd=3, main=paste(variable, " vs. pseudoausencias", sep=""), xlab=variable, ylim=c(0, max(d0$y, d1$y)))
lines(d1, col="forestgreen", lwd=3)
}

dev.off()



####################################
#DOS VARIABLES
####################################

#creando una tabla temporal para hacer el plot
tabla.visualizacion<-presencia.background
#la reordenamos para que las presencias nos aparezcan encima del backgraound
tabla.visualizacion<-tabla.visualizacion[order(tabla.visualizacion$presencia, decreasing=FALSE), ]
#colores
tabla.visualizacion$color<-"gray60"
tabla.visualizacion[which(tabla.visualizacion$presencia==1), "color"]<-"red4" 
#diferente tamaño para los puntos
tabla.visualizacion$cex<-0.6
tabla.visualizacion[which(tabla.visualizacion$presencia==1), "cex"]<-0.8

#ploteando los scatterplots
par(mfrow=c(2,2), mar=c(4,4,2,2))
plot(tabla.visualizacion$bio5 ~ tabla.visualizacion$bio6, col=tabla.visualizacion$color, cex=tabla.visualizacion$cex, ylab="bio5", xlab="bio6", pch=20)
plot(tabla.visualizacion$bio5 ~ tabla.visualizacion$bio14, col=tabla.visualizacion$color, cex=tabla.visualizacion$cex, ylab="bio5", xlab="bio14", pch=20)
plot(tabla.visualizacion$landcover_veg_tree ~ tabla.visualizacion$ndvi_range, col=tabla.visualizacion$color, cex=tabla.visualizacion$cex, ylab="landcover_veg_tree", xlab="ndvi_range", pch=20)
plot(tabla.visualizacion$human_footprint ~ tabla.visualizacion$diversidad_topo, col=tabla.visualizacion$color, cex=tabla.visualizacion$cex, ylab="human_footprint", xlab="diversidad_topo", pch=20)


##########################################
#NON METRIC MULTIDIMENSIONAL SCALING (NMDS)
##########################################
#USAMOS PSEUDOAUSENCIAS (porque el NMDS va muy lento si hay muchos puntos)
tabla.visualizacion<-presencia.pseudoausencia
#reordenamos para que las presencias se nos dibujen encima de las pseudoausencias
tabla.visualizacion<-tabla.visualizacion[order(tabla.visualizacion$presencia, decreasing=FALSE), ]
#colores
tabla.visualizacion$color<-"gray60"
tabla.visualizacion[which(tabla.visualizacion$presencia==1), "color"]<-"red4" 
#tamaño diferente para los puntos
tabla.visualizacion$cex<-1
tabla.visualizacion[which(tabla.visualizacion$presencia==1), "cex"]<-0.8

#NMDS
nmds<-metaMDS(tabla.visualizacion[,names(variables)], distance="euclidean")
#reglas para interpretar el stress
#stress < 0.05: excelente representación
#stress < 0.1: buena
#stress < 0.2: aceptable
#stress > 0.3: no satisfactoria

#PLOT
# pdf("./results/nmds.pdf", width=10, height=7, pointsize=15)
x11()
plot(nmds, type="n", main="NMDS")
points(nmds$points, col=tabla.visualizacion$color, cex=tabla.visualizacion$cex, pch=19)
ordisurf(nmds, tabla.visualizacion$ndvi_range, add=T, col="blue", lwd=2, cex=3)
ordisurf(nmds, tabla.visualizacion$bio5, add=T, col="forestgreen", lwd=2, cex=3)
legend("bottomright", c("pseudoausencia","presencia"), pch=19, col=c("gray60","red4"))
legend("topright", title="Lines", c("ndvi_average", "Bio5"), lty=1, lwd=2, col=c("blue", "forestgreen"))
# dev.off()

#eliminamos objetos que ya no necesitamos
rm(nmds, variable, variable.max, variable.min, d0, d1, tabla.visualizacion)
gc()


#################################################################
#################################################################
#################################################################
#CALIBRANDO MODELOS
#################################################################
#################################################################
#################################################################

#NUESTRO PRIMER MODELO DE LA A A LA Z
#####################################
#####################################

#VER SECCIÓN DE BIOCLIM EN LAS DIAPOSITIVAS


#1.- CALIBRANDO EL MODELO
#####################################
#calibramos un modelo BIOCLIM que solo requiere presencias
#NOTA IMPORTANTE, VAMOS A CALIBRAR TODOS LOS MODELOS CON LAS TABLAS presencia.*.entrenamiento, PORQUE LOS EVALUAREMOS TODOS JUNTOS AL FINAL PARA COMPARARLOS
m.bioclim=bioclim(variables, presencia.entrenamiento[, c("x", "y")])

#vemos el objeto por dentro
str(m.bioclim)


#2.- EXAMINANDO EL MODELO
#####################################
#para plotear el modelo, a y b son los índices de las variables en names(variables).
#en VERDE las presencias dentro del hipercubo según el modelo.
#en ROJO las presencias fuera del hipercubo según el modelo.
#en AZUL el nicho óptimo según las variables a y b del plot.
#aunque las presencias en rojo estén dentro del nicho óptimo para unos determinados a y b, estarán fuera para otros.
x11()
par(mfrow=c(2,2), mar=c(4,4,3,3))
plot(m.bioclim, a=1, b=2)
plot(m.bioclim, a=1, b=3)
plot(m.bioclim, a=1, b=4)
plot(m.bioclim, a=1, b=5)


#3.- PREDICIENDO EL MODELO A UN MAPA
#####################################
m.bioclim.map=dismo::predict(variables, m.bioclim)

#le ponemos nombre al mapa
names(m.bioclim.map)<-"bioclim"

#ploteamos el mapa
x11()
plot(m.bioclim.map)
points(presencia[, c("x","y")], cex=0.1)


#4.-EVALUANDO EL MODELO
#####################################
#extraemos los valores de las presencias y ausencias de evaluación (40% del total de presencias y ausencias)
valores.presencias<-extract(m.bioclim.map, presencia.ausencia.evaluacion[presencia.ausencia.evaluacion$presencia==1, c("x","y")])

valores.ausencias<-extract(m.bioclim.map, presencia.ausencia.evaluacion[presencia.ausencia.evaluacion$presencia==0, c("x","y")])

#aplicamos la función evaluate de la librería dismo
evaluacion<-evaluate(p=valores.presencias, a=valores.ausencias)
evaluacion

#ploteamos evaluacion
x11()
par(mfrow=c(2,2))
plot(evaluacion, 'ROC')
plot(evaluacion, 'TPR')
boxplot(evaluacion)
density(evaluacion)

#TODOS LOS MODELOS QUE VAMOS A HACER SIGUEN MÁS O MENOS LA MISMA ESTRUCTURA, PERO NO VAMOS A HACER LA EVALUACIÓN UNO A UNO PORQUE LA HAREMOS PARA TODOS JUNTOS AL FINAL. POR ESO GUARDAMOS ESTE MODELO EN UN BRICK LLAMADO modelos. NO BORRES NI OLVIDES ESTE BRICK, LO VAMOS A IR LLENANDO A LO LARGO DE LA SESIÓN.
modelos.mapas<-brick(m.bioclim.map)

#TAMBIÉN GUARDAMOS LOS OBJETOS DE LOS MODELOS EN UNA LISTA
modelos<-list()
modelos$bioclim<-m.bioclim

#BORRAMOS OBJETOS QUE NO NECESITAMOS
rm(evaluacion, m.bioclim, m.bioclim.map, valores.ausencias, valores.presencias)
gc()
 
################################
################################
#GENERALIZED LINEAR MODELS (GLM)
################################
################################

#VER SECCIÓN DE GLM EN LAS DIAPOSITIVAS

#AYUDA GLM
help(glm)

#EMPEZAMOS POCO A POCO, SOLO CON DOS VARIABLES
################################
m.glm.temp<-glm(presencia ~ bio5 + ndvi_range, family=binomial(link=logit), data=presencia.pseudoausencia.entrenamiento)
#family=binomial(link=logit) -> ESTAMOS HACIENDO REGRESIÓN LOGÍSTICA!

#RESUMEN DE RESULTADOS
summary(m.glm.temp)
#NOTA: los coeficientes no se pueden interpretar como medidas relativas de influencia si las variables no están en la misma escala.

#COEFICIENTES ESTANDARIZADOS
#QUÉ VARIABLE TIENE UN EFECTO MAYOR? ES POSITIVO O NEGATIVO EL EFECTO?
lm.beta(m.glm.temp)

#LO VEMOS CON LAS CURVAS DE RESPUESTA
plotmo(m.glm.temp, level=0.68, all2=TRUE)
#CUIDADO!: EL PLOT 3D NO SE PUEDE INTERPRETAR COMO UNA INTERACCIÓN PORQUE EL MODELO NO TIENE INTERACCIONES DEFINIDAS.

#DEVIANZA EXPLICADA
Dsquared(m.glm.temp)

#PREDECIMOS A UN MAPA
m.glm.temp.mapa<-predict(variables, m.glm.temp, type="response")

#dibujamos mapa
plot(m.glm.temp.mapa)
points(presencia[, c("x","y")], cex=0.1)


#AHORA CON DOS VARIABLES QUE INTERACCIONAN
################################
m.glm.temp<-glm(presencia ~ bio5 * ndvi_range, family=binomial(link=logit), data=presencia.pseudoausencia.entrenamiento)

#modelo
summary(m.glm.temp)
lm.beta(m.glm.temp)
Dsquared(m.glm.temp)
plotmo(m.glm.temp, level=0.68, all2=TRUE)

#mapa
m.glm.temp.mapa<-predict(variables, m.glm.temp, type="response")
plot(m.glm.temp.mapa)
points(presencia[, c("x","y")], cex=0.1)


#AHORA CON DOS VARIABLES CON TRANSFORMACIÓN POLINOMIAL QUE INTERACCIONAN (grado 2)
################################
m.glm.temp<-glm(presencia ~ poly(bio5, 2, raw=TRUE) * poly(ndvi_range, 2, raw=TRUE), family=binomial(link=logit), data=presencia.pseudoausencia.entrenamiento)

#modelo
summary(m.glm.temp)
lm.beta(m.glm.temp)
Dsquared(m.glm.temp)
plotmo(m.glm.temp, level=0.68, all2=TRUE)

#mapa
m.glm.temp.mapa<-predict(variables, m.glm.temp, type="response")
plot(m.glm.temp.mapa)
points(presencia[, c("x","y")], cex=0.1)


#AHORA CON DOS VARIABLES CON TRANSFORMACIÓN POLINOMIAL (grado 4)
################################
m.glm.temp<-glm(presencia ~ poly(bio5, 4, raw=TRUE) * poly(ndvi_range, 4, raw=TRUE), family=binomial(link=logit), data=presencia.pseudoausencia.entrenamiento)

#modelo
summary(m.glm.temp)
lm.beta(m.glm.temp)
Dsquared(m.glm.temp)
plotmo(m.glm.temp, level=0.68)

#mapa
m.glm.temp.mapa<-predict(variables, m.glm.temp, type="response")
plot(m.glm.temp.mapa)
points(presencia[, c("x","y")], cex=0.1)


#VAMOS A TRABAJAR CON TODAS LAS VARIABLES
################################
#GENERA FÓRMULAS PARA LOS MODELOS DE REGRESIÓN
#formula de regresión polinomial de grado 2 sin interacciones
formula.regresion.poly<-as.formula(paste("presencia ~ poly(", paste(names(variables), collapse=", 2, raw=TRUE) + poly("), ", 2, raw=TRUE)", collapse=""))
formula.regresion.poly

#NOTA IMPORTANTE: VAMOS A CALIBRAR CADA MODELO CON AUSENCIAS, PSEUDO-AUSENCIAS Y BACKGROUND, PARA APRENDER COMO FUNCIONA CADA ALGORITMO CON ELLAS

#BACKGROUND
m.glm.background<-glm(formula.regresion.poly, family=binomial(link=logit), data=presencia.background.entrenamiento)

#BACKGROUND PONDERADO
pesos<-WeightPresenceBackground(presencia.background.entrenamiento[ , "presencia"])
#la función WeightPresenceBackground cuenta el número de presencias, el número de puntos de background, divide 1 por cada uno de ellos, y genera un vector con los valores resultantes
#para casos ponderados ponemos family=quasibinomial
m.glm.backgroundw<-glm(formula.regresion.poly, family=quasibinomial(link=logit), data=presencia.background.entrenamiento, weights=pesos)

#AUSENCIA
m.glm.ausencia<-glm(formula.regresion.poly, family=binomial(link=logit), data=presencia.ausencia.entrenamiento)

#PSEUDOAUSENCIA
m.glm.pseudoausencia<-glm(formula.regresion.poly, family=binomial(link=logit), data=presencia.pseudoausencia.entrenamiento)


#CURVAS DE RESPUESTA
help(plotmo)
#fíjate en las diferencias entre las curvas de respuesta de los modelos de background
x11()
plotmo(m.glm.background, level=0.68)
x11()
plotmo(m.glm.backgroundw, level=0.68)
x11()
plotmo(m.glm.ausencia, level=0.68)
x11()
plotmo(m.glm.pseudoausencia, level=0.68)

#RESULTADOS DE LOS MODELOS
summary(m.glm.background)
summary(m.glm.backgroundw)
summary(m.glm.ausencia)
summary(m.glm.pseudoausencia)

#COEFICIENTES ESTANDARIZADOS
lm.beta(m.glm.background)
lm.beta(m.glm.backgroundw)
lm.beta(m.glm.ausencia)
lm.beta(m.glm.pseudoausencia)

#DEVIANZA EXPLICADA
Dsquared(m.glm.background)
Dsquared(m.glm.backgroundw)
Dsquared(m.glm.ausencia)
Dsquared(m.glm.pseudoausencia)

#MAPAS
m.glm.background.mapa<-predict(variables, m.glm.background, type="response")
m.glm.backgroundw.mapa<-predict(variables, m.glm.backgroundw, type="response")
m.glm.ausencia.mapa<-predict(variables, m.glm.ausencia, type="response")
m.glm.pseudoausencia.mapa<-predict(variables, m.glm.pseudoausencia, type="response")

#NOMBRES DE LOS MAPAS
names(m.glm.background.mapa)<-"glm.background"
names(m.glm.backgroundw.mapa)<-"glm.background.ponderado"
names(m.glm.ausencia.mapa)<-"glm.ausencia"
names(m.glm.pseudoausencia.mapa)<-"glm.pseudoausencia"

#guardamos mapas
modelos.mapas<-addLayer(modelos.mapas, m.glm.background.mapa, m.glm.backgroundw.mapa, m.glm.ausencia.mapa, m.glm.pseudoausencia.mapa)

#guardamos los modelos entrenados
modelos$glm.background<-m.glm.background
modelos$glm.background.ponderado<-m.glm.backgroundw
modelos$glm.ausencia<-m.glm.ausencia
modelos$glm.pseudoausencia<-m.glm.pseudoausencia



#PLOT
par(mfrow=c(2,2), oma=c(0,0,0,0), mar=c(1,1,1,1))
plot(m.glm.background.mapa, main="background")
plot(m.glm.backgroundw.mapa, main="background ponderado")
plot(m.glm.ausencia.mapa, main="absence")
plot(m.glm.pseudoausencia.mapa, main="pseudoausencia")



#################################################
#AVANZADO: SELECCIÓN AUTOMÁTICA DE MODELOS CON glmulti
#################################################
#AJUSTAMOS MODELO CON PSEUDOAUSENCIAS (más rápido)
m.glm.temp<-glm(formula.regresion.poly, family=binomial(link="logit"), data=presencia.pseudoausencia)

#selección de modelos (aquí hay muchas opciones, invéstigalas leyendo la ayuda de la función glmulti)
#method="g" implica que usamos un algoritmo genético. "h" para examinar todos los posibles modelos (tarda mucho más).
#level=1 sin interacciones, level=2 con interacciones (tarda mucho más).
m.glm.temp.seleccion<-glmulti(m.glm.temp, level=1, method="g")

#MODELO
summary(m.glm.temp.seleccion)

#FORMULA DEL MEJOR MODELO
formula.mejor.modelo=as.formula(summary(m.glm.temp.seleccion)$bestmodel)
formula.mejor.modelo

#AJUSTAMOS MODELO CON LA NUEVA FÓRMULA
m.glm.temp<-glm(formula.mejor.modelo, family=binomial(link="logit"), data=presencia.pseudoausencia)

#modelo
summary(m.glm.temp)
lm.beta(m.glm.temp)
Dsquared(m.glm.temp)
plotmo(m.glm.temp, level=0.68)

#mapa
m.glm.temp.mapa<-predict(variables, m.glm.temp, type="response")
names(m.glm.temp.mapa)<-"glm.pseudoausencia.glmulti"

dev.off()
x11()
plot(m.glm.temp.mapa)
points(presencia[, c("x","y")], cex=0.1)

#lo añadimos al brick "modelos"
modelos.mapas<-addLayer(modelos.mapas, m.glm.temp.mapa)
modelos$glm.pseudoausencia.glmulti<-m.glm.temp

#BORRAMOS OBJETOS QUE NO NECESITAMOS
rm(formula.regresion.poly, m.glm.ausencia, m.glm.ausencia.mapa, m.glm.background, m.glm.background.mapa, m.glm.backgroundw, m.glm.backgroundw.mapa, m.glm.pseudoausencia, m.glm.pseudoausencia.mapa, m.glm.temp.mapa, m.glm.temp, pesos, m.glm.temp.seleccion, formula.mejor.modelo)
gc()


#####################################
#####################################
#MODELOS ADITIVOS GENERALIZADOS (GAM)
#####################################
#####################################

#AJUSTANDO UN MODELO
##################################################
#FORMULA
formula.gam<-as.formula(paste("presencia ~ s(", paste(names(variables), collapse=") + s("), ")", collapse=""))
formula.gam
#Fíjate que requiere encapsular las variables en una función llamada "s()"
#Esto es un requerimiento de los modelos GAM, 

#calibramos modelo (con pseudoausencias porque es más rápido)
#select=TRUE cada término puede ser penalizado a 0 si no aporta nada al modelo.
m.gam.temp<-gam(formula.gam, family=binomial(link=logit), data=presencia.pseudoausencia.entrenamiento, select=TRUE)

#curvas de respuesta
x11()
plotmo(m.gam.temp, level=0.68, type="response")


#SIMPLIFICANDO EL MODELO
##################################################
#k define la complejidad de las funciones ajustadas para cada una de las variables.
k<-3

#GENERA LA FÓRMULA PARA GAM
formula.gam.k3<-as.formula(paste("presencia ~ s(", paste(names(variables), ", k=", k,  collapse=") + s("), ")", collapse=""))
formula.gam.k3

#comparamos con la anterior
formula.gam

#ajustamos modelo
m.gam.temp.k3<-gam(formula.gam.k3, family=binomial(link=logit), data=presencia.pseudoausencia.entrenamiento, select=TRUE)

#curvas de respuesta
x11()
plotmo(m.gam.temp.k3, level=0.68, type="response")

#COMPARANDO LOS MODELOS
######################
#resumen de los modelos
summary(m.gam.temp)
summary(m.gam.temp.k3)

#para encontrar los valores a extraer
str(summary(m.gam.temp))

#R cuadrado
summary(m.gam.temp)$r.sq
summary(m.gam.temp.k3)$r.sq

#devianza explicada
summary(m.gam.temp)$dev.expl
summary(m.gam.temp.k3)$dev.expl

#UBRE - UnBiaed Risk Estimator
#INTERPRETACIÓN: valores más pequeños indican mejores modelos
summary(m.gam.temp)$sp.criterion
summary(m.gam.temp.k3)$sp.criterion

#TAMBIÉN PODEMOS CALCULAR EL AKAIKE INFORMATION CRITERION
#INTERPRETACIÓN: valores más pequeños indican mejores modelos
AIC(m.gam.temp, m.gam.temp.k3)

#VAMOS A METER AMBOS MODELOS EN EL BRICK modelos
m.gam.temp.mapa<-predict(variables, m.gam.temp, type="response")
m.gam.temp.k3.mapa<-predict(variables, m.gam.temp.k3, type="response")
names(m.gam.temp.mapa)<-"gam.pseudoausencia"
names(m.gam.temp.k3.mapa)<-"gam.pseudoausencia.k3"
modelos.mapas<-addLayer(modelos.mapas, m.gam.temp.mapa, m.gam.temp.k3.mapa)

#guardamos los objetos de los modelos
modelos$gam.pseudoausencia<-m.gam.temp
modelos$gam.pseudoausencia.k3<-m.gam.temp.k3

#y los ploteamos
x11()
par(mfrow=c(1,2))
plot(m.gam.temp.mapa)
plot(m.gam.temp.k3.mapa)

##############################################################
#AVANZADO: QUÉ VALORES DE K SON MÁS ADECUADOS?: ANÁLISIS DE SENSIBILIDAD
##############################################################
#Cuales son los mejores valores de k?
#1.- Iteramos sobre distintos valores de k para cada una de las variables
#2.- Almacemamos los valores de devianza explicada, r-squared, y UBRE para cada valor
#3.- Ploteamos el resultado y decidimos

#creamos vectores para guardar resultados
k.interval<-3:40
variable.vector<-vector()
k.vector<-vector() #de 5 a 50 de 5 en 5
r.sq.vector<-vector()
dev.expl.vector<-vector()
sp.criterion.vector<-vector()

#iteramos sobre cada una de las variables
for(variable in names(variables)){

  #iteramos sobre distintos valores de k
  for(k in k.interval){
    
    #generamos formula
    formula.gam.k<-as.formula(paste("presencia ~ s(", paste(variable, ", k=", k,  collapse=") + s("), ")", collapse=""))
    
    #ajustamos modelo
    m.gam.temp.k<-gam(formula.gam.k, family=binomial(link=logit), data=presencia.pseudoausencia.entrenamiento, select=TRUE)
    
    #llenamos vectores valores
    variable.vector<-c(variable.vector, paste(variable))
    k.vector<-c(k.vector, k)
    r.sq.vector<-c(r.sq.vector, summary(m.gam.temp.k)$r.sq)
    dev.expl.vector<-c(dev.expl.vector, summary(m.gam.temp.k)$dev.expl)
    sp.criterion.vector<-c(sp.criterion.vector, summary(m.gam.temp.k)$sp.criterion)
    
  }#final de la iteración sobre los k
  
}#final de la iteración sobre las variables

#ponemos los datos juntos en un dataframe
sensibilidad<-data.frame(k=k.vector, variable=variable.vector, r.sq=r.sq.vector, dev.expl=dev.expl.vector, sp.criterion=sp.criterion.vector)

#ploteamos
plot.sp.criterion<-ggplot(data=sensibilidad, aes(x=k, y=sp.criterion, color=variable)) + geom_line()
plot.dev.expl<-ggplot(data=sensibilidad, aes(x=k, y=dev.expl, color=variable)) + geom_line()
grid.arrange(plot.sp.criterion, plot.dev.expl, nrow=2)

#buscamos los valores de k optimos
best.k<-vector()

#iteramos por las variables
for (variable in names(variables)){
  
  #seleccionamos los datos de esa variable en subset
  temp<-sensibilidad[sensibilidad$variable==variable,]
  
  #seleccionamos la k del primer registro con mínimo sp.criterion
  best.k<-c(best.k, temp[which.min(temp$sp.criterion),"k"])
}

#generamos la nueva formula con best.k
formula.gam.best.k<-as.formula(paste("presencia ~ s(", paste(names(variables), ", k=", best.k,  collapse=") + s(", sep=""), ")", collapse="", sep=""))

#ajustamos el modelo
m.gam.temp.best.k<-gam(formula.gam.best.k, family=binomial(link=logit), data=presencia.pseudoausencia.entrenamiento, select=TRUE)

#ploteamos curvas de respuesta
plotmo(m.gam.temp.best.k, level=0.68, type="response", caption="Best k")

#lo comparamos con los otros
#R cuadrado
summary(m.gam.temp)$r.sq
summary(m.gam.temp.k3)$r.sq
summary(m.gam.temp.best.k)$r.sq

#devianza explicada
summary(m.gam.temp)$dev.expl
summary(m.gam.temp.k3)$dev.expl
summary(m.gam.temp.best.k)$dev.expl

#UBRE
summary(m.gam.temp)$sp.criterion
summary(m.gam.temp.k3)$sp.criterion
summary(m.gam.temp.best.k)$sp.criterion

#AIC
AIC(m.gam.temp, m.gam.temp.k3, m.gam.temp.best.k)
#CONCLUSION: el modelo con best k es el que tiene el mejor ajuste según todas las métricas

#VAMOS A CONVERTIRLO EN UN MAPA Y GUARDARLO, PARA EVALUARLO MÁS TARDE
m.gam.best.k.mapa<-predict(variables, m.gam.temp.best.k, type="response")
names(m.gam.best.k.mapa)<-"gam.pseudoausencia.best.k"
#lo añadimos a modelos
modelos.mapas<-addLayer(modelos.mapas, m.gam.best.k.mapa)

#guardamos el objeto
modelos$gam.pseudoausencia.best.k<-m.gam.temp.best.k

#plot
x11()
plot(m.gam.best.k.mapa)
############################################################################################


#GENERAMOS MODELOS CON LOS DISTINTOS FICHEROS DE PRESENCIA Y EL k USADO POR DEFECTO POR LA FUNCIÓN s()
##########################################################################################

#BACKGROUND
m.gam.background<-gam(formula.gam, family=binomial(link=logit), data=presencia.background.entrenamiento, select=TRUE)

#BACKGROUND PONDERADO ojo a weights=pesos
pesos<-WeightPresenceBackground(presencia.background.entrenamiento[ , "presencia"])
m.gam.backgroundw<-gam(formula.gam, family=quasibinomial(link=logit), data=presencia.background.entrenamiento, weights=pesos, select=TRUE)

#AUSENCIA
m.gam.ausencia<-gam(formula.gam, family=binomial(link=logit), data=presencia.ausencia.entrenamiento, select=TRUE)

#PSEUDO AUSENCIA (YA LO TENEMOS!)
# m.gam.pseudoausencia<-gam(formula.gam, family=binomial(link=logit), data=presencia.pseudoausencia.entrenamiento, select=TRUE)
m.gam.pseudoausencia<-m.gam.temp

#DIBUJAMOS LAS CURVAS DE RESPUESTA
x11()
plotmo(m.gam.background, level=0.68, type="response", caption="Background")
x11()
plotmo(m.gam.backgroundw, level=0.68, caption="Background ponderado")
x11()
plotmo(m.gam.ausencia, level=0.68, type="response", caption="Ausencia")
x11()
plotmo(m.gam.pseudoausencia, level=0.68, type="response", caption="Pseudo-ausencia")

#RESUMEN
summary(m.gam.background)
summary(m.gam.backgroundw)
summary(m.gam.ausencia)
summary(m.gam.pseudoausencia)

#DEVIANZA
summary(m.gam.background)$dev.expl
summary(m.gam.backgroundw)$dev.expl
summary(m.gam.ausencia)$dev.expl
summary(m.gam.pseudoausencia)$dev.expl

#UBRE
summary(m.gam.background)$sp.criterion
summary(m.gam.backgroundw)$sp.criterion
summary(m.gam.ausencia)$sp.criterion
summary(m.gam.pseudoausencia)$sp.criterion

#AIC
#NOTA: AIC no se puede calcular para m.gam.backgroundw porque la familia con la que hacemos el modelo no es binomial
AIC(m.gam.background, m.gam.ausencia, m.gam.pseudoausencia)

#PREDICCION GEOGRAFICA
m.gam.background.mapa<-predict(variables, m.gam.background, type="response")
m.gam.backgroundw.mapa<-predict(variables, m.gam.backgroundw, type="response")
m.gam.ausencia.mapa<-predict(variables, m.gam.ausencia, type="response")
m.gam.pseudoausencia.mapa<-predict(variables, m.gam.pseudoausencia, type="response")

#NOMBRES DE LOS MAPAS
names(m.gam.background.mapa)<-"gam.background"
names(m.gam.backgroundw.mapa)<-"gam.background.ponderado"
names(m.gam.ausencia.mapa)<-"gam.ausencia"
# names(m.gam.pseudoausencia.mapa)<-"gam.pseudoausencia" #no lo necesitamos, ya está en el brick

#MAPAS
x11()
par(mfrow=c(2,2), oma=c(0,0,0,0), mar=c(1,1,1,1))
plot(m.gam.background.mapa, main="background")
plot(m.gam.backgroundw.mapa, main="background ponderado")
plot(m.gam.ausencia.mapa, main="absence")
plot(m.gam.pseudoausencia.mapa, main="pseudoausencia")

#ESCRIBE A modelos
modelos.mapas<-addLayer(modelos.mapas, m.gam.background.mapa, m.gam.backgroundw.mapa, m.gam.ausencia.mapa)

#guardamos los objetos
modelos$gam.background<-m.gam.background
modelos$gam.background.ponderado<-m.gam.backgroundw
modelos$gam.ausencia<-m.gam.ausencia


#borra el mapa y el modelo de la memoria
rm(m.gam.ausencia, m.gam.ausencia.mapa, m.gam.background, m.gam.background.mapa, m.gam.backgroundw, m.gam.backgroundw.mapa, m.gam.pseudoausencia, m.gam.pseudoausencia.mapa, pesos, formula.gam, best.k, formula.gam.k, k, k.vector, m.gam.temp.best.k, m.gam.temp.k3, plot.dev.expl, dev.expl.vector, formula.gam.best.k, formula.gam.k3, k.interval, m.gam.best.k.mapa, m.gam.temp, m.gam.temp.k, plot.sp.criterion, r.sq.vector, sp.criterion.vector, variable, variable.vector, sensibilidad, temp, formula.gam.best.k, m.gam.temp.k3.mapa, m.gam.temp.mapa)
gc()


#####################################
#####################################
#MAXENT
#####################################
#####################################

#VER SECCIÓN MAXENT EN LAS DIAPOSITIVAS

#ajustamos el modelo con sus argumentos por defecto
m.maxent=maxnet(p=presencia.background$presencia, data=presencia.background[, names(variables)])

#qué hay dentro del modelo?
str(m.maxent)

#coeficientes de los predictores, combinaciones lineales de los predictores, y transformaciones de los predictores.
sort(abs(m.maxent$betas)[1:10], decreasing=TRUE)

#curvas de respuesta
x11()
plot(m.maxent, type="logistic")

#map
maxent.model.map<-predict(variables, m.maxent, type="logistic")
names(maxent.model.map)<-"maxent"
modelos.mapas<-addLayer(modelos.mapas, maxent.model.map)

#plot del modelo
x11()
plot(maxent.model.map)

#guardamos el objeto
modelos$maxent<-m.maxent

#MAXENT TIENE UN PARÁMETRO PRAA CONTROLAR LA COMPLEJIDAD DEL MODELO: "regmult"
#COMO ELEGIMOS EL VALOR MÁS ADECUADO? ANÁLISIS DE SENSIBILIDAD!

#vectores para los valores que necesitamos
regmult.vector<-1:10
auc.vector<-vector()

#iteramos sobre valores de regmult
for (regmult.valor in regmult.vector){

  #ajustamos el modelo con el valor de regmult apropiado
  m.maxent.temp=maxnet(p=presencia.background$presencia, data=presencia.background[, names(variables)], regmult=regmult.valor)
  
  #evaluamos el modelo y guardamos el valor de evaluación en auc.vector
  auc.vector<-c(auc.vector, evaluate(
    p=presencia.ausencia.entrenamiento[presencia.ausencia.entrenamiento$presencia==1, c("x", "y")], 
    a=presencia.ausencia.entrenamiento[presencia.ausencia.entrenamiento$presencia==0, c("x", "y")], 
    model=m.maxent.temp, 
    x=variables
    )@auc)
  
}#final de las iteraciones

#vemos el resultado
plot(regmult.vector, auc.vector, type="l", xlab="regmult", ylab="auc")
#ESTÁ CLARO QUE LA CONFIGURACIÓN POR DEFECTO ES LA MEJOR EN ESTE CASO, NOS QUEDAMOS CON EL MODELO ENTRENADO CON regmult=1 (QUE ES EL PRIMERO QUE HICIMOS)

#BORRAMOS COSAS QUE YA NO NECESITAMOS
rm(regmult.valor, regmult.vector, m.maxent, m.maxent.temp, maxent.model.map, auc.vector)
gc()

############################################
############################################
#MULTI-ADAPTIVE REGRESSION SPLINES (MARS)
############################################
############################################
#EARTH = Enhanced Adaptive Regression Through Hinges”
#VER SECCIÓN MARS EN LAS DIAPOSITIVAS

#FORMULA
formula.mars<-as.formula(paste("presencia ~ ", paste(names(variables), collapse="+")))
formula.mars

#BACKGROUND
m.mars.background<-earth(formula.mars, glm = list(family=binomial(link="logit")), data=presencia.background)

#background ponderado NO LO HACEMOS, DEMASIADO LENTO
# m.mars.backgroundw<-earth(formula.mars, glm = list(family=quasibinomial(link="logit")), data=presencia.background, weights=WeightPresenceBackground(presencia.background[ , "presencia"]), Use.beta.cache=TRUE)

#AUSENCIA
m.mars.ausencia<-earth(formula.mars, glm = list(family=binomial(link="logit")), data=presencia.ausencia)

#PSEUDOAUSENCIA
m.mars.pseudoausencia<-earth(formula.mars, glm = list(family=binomial(link="logit")), data=presencia.pseudoausencia)

#CURVAS DE RESPUESTA
x11()
plotmo(m.mars.background, type="response", caption="background", all2=TRUE)
# x11()
# plotmo(m.mars.backgroundw, type="response", caption="background ponderado", all2=TRUE)
x11()
plotmo(m.mars.ausencia, type="response", caption="ausencia", all2=TRUE)
x11()
plotmo(m.mars.pseudoausencia, type="response", caption="pseudoausencia", all2=TRUE)

#summary
summary(m.mars.background)
# summary(m.mars.backgroundw)
summary(m.mars.ausencia)
summary(m.mars.pseudoausencia)

#variable importance
#nsubsets, gcv y rss son medidas de la importancia de la variable
#nsubsets cuenta el número de componentes del modelo que incluyen la variable. Las variables que están en más componentes del modelo se consideran más importantes.
#rss: residual sum-of-squares
#gcv: generalized cross-validation 
evimp(m.mars.background)
#evimp(m.mars.backgroundw)
evimp(m.mars.ausencia)
evimp(m.mars.pseudoausencia)

#MAPAS
m.mars.background.map<-predict(variables, m.mars.background, type="response")
# m.mars.backgroundw.map<-predict(variables, m.mars.backgroundw, type="response")
m.mars.ausencia.map<-predict(variables, m.mars.ausencia, type="response")
m.mars.pseudoausencia.map<-predict(variables, m.mars.pseudoausencia, type="response")

#NOMBRES DE LOS MAPAS
names(m.mars.background.map)<-"mars.background"
# names(m.mars.backgroundw.map)<-"mars.background.ponderado"
names(m.mars.ausencia.map)<-"mars.ausencia"
names(m.mars.pseudoausencia.map)<-"mars.pseudoausencia"

#maps
x11()
par(mfrow=c(2,2))
plot(m.mars.background.map, main="background")
# plot(m.mars.backgroundw.map, main="background ponderado")
plot(m.mars.ausencia.map, main="ausencia")
plot(m.mars.pseudoausencia.map, main="pseudo-ausencia")

#LOS GUARDAMOS EN modelos
modelos.mapas<-addLayer(modelos.mapas, m.mars.background.map, m.mars.ausencia.map, m.mars.pseudoausencia.map)

#guardamos los objetos
modelos$mars.background<-m.mars.background
modelos$mars.ausencia<-m.mars.ausencia
modelos$mars.pseudoausencia<-m.mars.pseudoausencia

#BORRAMOS COSAS QUE NO NECESITAMOS
rm(m.mars.ausencia, m.mars.background, m.mars.pseudoausencia, formula.mars, m.mars.ausencia.map, m.mars.background.map, m.mars.pseudoausencia.map)


#####################################
#####################################
#REGRESSION TREES
#####################################
#####################################
#la formula requerida es igual que la que usamos para MARS o el GLM sin transformación polinomial
formula.tree<-as.formula(paste("presencia ~ ", paste(names(variables), collapse="+")))
formula.tree

#ajustando árboles de regresión con la librería "tree"
m.tree.background<-tree(formula.tree, data=presencia.background)
m.tree.pseudoausencia<-tree(formula.tree, data=presencia.pseudoausencia)
m.tree.ausencia<-tree(formula.tree, data=presencia.ausencia)

#dibujamos los árboles
x11()
par(mfrow=c(1,3))
plot(m.tree.background)
text(m.tree.background, all=TRUE, cex=1)
plot(m.tree.pseudoausencia)
text(m.tree.pseudoausencia, all=TRUE, cex=1)
plot(m.tree.ausencia)
text(m.tree.ausencia, all=TRUE, cex=1)

#prediciendo a un mapa
m.tree.background.map<-predict(variables, m.tree.background)
m.tree.pseudoausencia.map<-predict(variables, m.tree.pseudoausencia)
m.tree.ausencia.map<-predict(variables, m.tree.ausencia)

#nombres
names(m.tree.background.map)<-"tree.background"
names(m.tree.pseudoausencia.map)<-"tree.pseudoausencia"
names(m.tree.ausencia.map)<-"tree.ausencia"

#los guardamos en "modelos"
modelos.mapas<-addLayer(modelos.mapas, m.tree.background.map, m.tree.pseudoausencia.map, m.tree.ausencia.map)

#maps
x11()
par(mfrow=c(1,3), oma=c(0,0,0,0))
plot(m.tree.background.map)
plot(m.tree.pseudoausencia.map)
plot(m.tree.ausencia.map)

#guardamos los objetos
modelos$tree.background<-m.tree.background
modelos$tree.pseudoausencia<-m.tree.pseudoausencia
modelos$tree.ausencia<-m.tree.ausencia

#borramos cosas que no necesitamos
rm(m.tree.ausencia.map, m.tree.background.map, m.tree.pseudoausencia.map, m.tree.ausencia, m.tree.background, m.tree.pseudoausencia)
gc()


#####################################
#####################################
#CONDITIONAL INFERENCE TREES
#####################################
#####################################
#USAN LA MISMA FŔOMULA
m.ctree.background<-ctree(formula.tree, data=presencia.background)
m.ctree.pseudoausencia<-ctree(formula.tree, data=presencia.pseudoausencia)
m.ctree.ausencia<-ctree(formula.tree, data=presencia.ausencia)

#ploteamos los modelos
x11()
plot(m.ctree.background)
x11()
plot(m.ctree.pseudoausencia)
x11()
plot(m.ctree.ausencia)

#predicción a mapa
m.ctree.background.map<-predict(variables, m.ctree.background)
m.ctree.pseudoausencia.map<-predict(variables, m.ctree.pseudoausencia)
m.ctree.ausencia.map<-predict(variables, m.ctree.ausencia)

#nombres
names(m.ctree.background.map)<-"ctree.background"
names(m.ctree.pseudoausencia.map)<-"ctree.pseudoausencia"
names(m.ctree.ausencia.map)<-"ctree.ausencia"

#los guardamos en "modelos"
modelos.mapas<-addLayer(modelos.mapas, m.ctree.background.map, m.ctree.pseudoausencia.map, m.ctree.ausencia.map)

#maps
x11()
par(mfrow=c(1,3), oma=c(0,0,0,0))
plot(m.ctree.background.map)
plot(m.ctree.pseudoausencia.map)
plot(m.ctree.ausencia.map)

#guardamos los objetos
modelos$ctree.background<-m.ctree.background
modelos$ctree.pseudoausencia<-m.ctree.pseudoausencia
modelos$ctree.ausencia<-m.ctree.ausencia

#borramos cosas que no necesitamos
rm(m.ctree.ausencia.map, m.ctree.background.map, m.ctree.pseudoausencia.map, m.ctree.ausencia, m.ctree.background, m.ctree.pseudoausencia, formula.tree)
gc()


#####################################
#####################################
#RANDOM FOREST
#####################################
#####################################
#AYUDA DE randomForest
help(randomForest)

#PREPARAMOS LA FÓRMULA
formula.regresion<-as.formula(paste("presencia ~ ", paste(names(variables), collapse="+"), collapse=""))
formula.regresion

#UN MODELO SENCILLO
######################################
m.rf.simple<-randomForest(formula.regresion, data=presencia.pseudoausencia.entrenamiento)

#vemos devianza explicada
print(m.rf.simple)

#curvas de respuesta
plotmo(m.rf.simple)

#dibujamos mapa
plot(predict(variables, m.rf.simple, type="response"))

#número de árboles
plot(m.rf.simple)

#UN MODELO CON MÁS ÁRBOLES
######################################
m.rf.simple<-randomForest(formula.regresion, data=presencia.pseudoausencia.entrenamiento, ntree=1000)

#número de árboles
plot(m.rf.simple)
plot(m.rf.simple, ylim=c(0.092, 0.10))


#IMPORTANCIA DE LAS VARIABLES
###################################################################
m.rf.simple<-randomForest(formula.regresion, data=presencia.pseudoausencia.entrenamiento, ntree=800, importance=TRUE)

#tabla de la importancia de las variables
importance(m.rf.simple)
varImpPlot(m.rf.simple)


#TAMAÑO DE NODO (nodesize)
###################################################################
m.rf.nodesize10<-randomForest(formula.regresion, data=presencia.pseudoausencia.entrenamiento, ntree=800, nodesize=10)

m.rf.nodesize100<-randomForest(formula.regresion, data=presencia.pseudoausencia.entrenamiento, ntree=800, nodesize=100)

#vemos devianza explicada
print(m.rf.nodesize10)
print(m.rf.nodesize100)

#mapas
x11()
par(mfrow=c(1,2))
breaks <- seq(0, 1, by=0.05) 
numero.colores <- length(breaks)-1 
colores <- rev(terrain.colors(numero.colores))
plot(predict(variables, m.rf.nodesize10, type="response"), breaks=breaks, col=colores, main="Nodesize 10")
plot(predict(variables, m.rf.nodesize100, type="response"), breaks=breaks, col=colores, main="Nodesize 100")

#EL DE nodesize=10 tiene un porcentaje de varianza explicada más alto, pero... se traduce eso en una capacidad predictiva mayor? Hacemos análisis de sensibilidad para averiguarlo.


#ANALISIS DE SENSIBILIDAD
##########################################
nodesize.vector<-seq(10, 200, by=10)
auc.vector<-vector()

#iteramos sobre valores de regmult
for (nodesize.valor in nodesize.vector){
  
  #ajustamos modelo
  m.rf.temp<-randomForest(formula.regresion, data=presencia.pseudoausencia.entrenamiento, ntree=800, nodesize=nodesize.valor)
  
  #evaluamos el modelo y guardamos el valor de evaluación en auc.vector
  auc.vector<-c(auc.vector, evaluate(
    p=presencia.ausencia.entrenamiento[presencia.ausencia.entrenamiento$presencia==1, c("x", "y")], 
    a=presencia.ausencia.entrenamiento[presencia.ausencia.entrenamiento$presencia==0, c("x", "y")], 
    model=m.rf.temp, 
    x=variables
  )@auc)
  
}#final de las iteraciones

#vemos el resultado
dev.off()
x11()
plot(nodesize.vector, auc.vector, type="l", xlab="nodesize", ylab="auc")
#CONCLUSIÓN: UN nodesize MENOR INCREMENTA LA CAPACIDAD PREDICTIVA DEL MODELO.

#HACEMOS UN MODELO CON CADA TIPO DE PRESENCIAS
###############################################

#BACKGROUND
m.rf.background<-randomForest(formula.regresion, data=presencia.background.entrenamiento, ntree=800, nodesize=10)

#NO ES POSIBLE PONDERAR CON PSEUDOAUSENCIAS EN RANDOM FOREST

#AUSENCIA
m.rf.ausencia<-randomForest(formula.regresion, data=presencia.ausencia.entrenamiento, ntree=800, nodesize=10)

#PSEUDO-AUSENCIA
m.rf.pseudoausencia<-randomForest(formula.regresion, data=presencia.pseudoausencia.entrenamiento, ntree=500, nodesize=10)


#DEVIANZA EXPLICADA
print(m.rf.background)
print(m.rf.ausencia)
print(m.rf.pseudoausencia)

#CURVAS DE RESPUESTA
x11()
plotmo(m.rf.background, caption="backgrouns")
x11()
plotmo(m.rf.ausencia, caption="ausencia")
x11()
plotmo(m.rf.pseudoausencia, caption="pseudoausencia")

#PREDICCIÓN GEOGRÁFICA
m.rf.background.mapa<-predict(variables, m.rf.background, type="response")
m.rf.ausencia.mapa<-predict(variables, m.rf.ausencia, type="response")
m.rf.pseudoausencia.mapa<-predict(variables, m.rf.pseudoausencia, type="response")

#NOMBRES Y AÑADIMOS A modelos
names(m.rf.background.mapa)<-"rf.background"
names(m.rf.ausencia.mapa)<-"rf.ausencia"
names(m.rf.pseudoausencia.mapa)<-"rf.pseudoausencia"
modelos.mapas<-addLayer(modelos.mapas, m.rf.background.mapa, m.rf.ausencia.mapa, m.rf.pseudoausencia.mapa)

#MAPA
x11()
par(mfrow=c(1,3))
plot(m.rf.background.mapa, main="background")
plot(m.rf.ausencia.mapa, main="ausencia")
plot(m.rf.pseudoausencia.mapa, main="pseudoausencia")

#guardamos los objetos
modelos$rf.background<-m.rf.background
modelos$rf.pseudoausencia<-m.rf.pseudoausencia
modelos$rf.ausencia<-m.rf.ausencia

#borra objetos que no nos sirven
rm(m.rf.ausencia, m.rf.ausencia.mapa, m.rf.background, m.rf.background.mapa, m.rf.pseudoausencia, m.rf.pseudoausencia.mapa, auc.vector, cols, breaks, formula.regresion, m.rf.nodesize10, m.rf.simple, m.rf.simple.nodesize100, m.rf.nodesize100, m.rf.simple.mapa, m.rf.temp, nodesize.valor, nodesize.vector, numero.colores, colores, auc.vector)
gc()



#####################################
#####################################
#BOOSTED REGRESSION TREES
#####################################
#####################################
#DOCUMENTO IMPORTANTE: https://cran.r-project.org/web/packages/dismo/vignettes/brt.pdf

#EL MODELO MÁS SIMPLE
#####################################
#NOTA: UNA learning.rate REALISTA SERÍA 0.005, PERO ESO HACE EL MODELO MUY LENTO PARA ESTA CLASE
#NOTA: gbm.step CALCULA EL NÚMERO ÓPTIMO DE ÁRBOLES
m.brt.simple=gbm.step(data=presencia.pseudoausencia, gbm.x = names(variables), gbm.y = "presencia", learning.rate=0.01)

#estadísticas de evaluación
m.brt.simple$cv.statistics
#auc
m.brt.simple$cv.statistics$discrimination.mean

#TARDA PORQUE gbm.step CALCULA EL NÚMERO ÓPTIMO DE ÁRBOLES DE REGRESIÓN USANDO VALIDACIÓN CRUZADA.

#plot error
plot(m.brt.simple)

#variable importance
summary(m.brt.simple)

#curvas de respuesta
gbm.plot(m.brt.simple)
plotmo(m.brt.simple)

#mapa (ojo con el argument n.trees!)
m.brt.simple.map<-predict(variables, m.brt.simple, n.trees=m.brt.simple$gbm.call$best.trees, type="response")
x11()
plot(m.brt.simple.map)

#interacciones entre variables
m.brt.simple.interacciones <- gbm.interactions(m.brt.simple)
#interaction matrix
m.brt.simple.interacciones$interactions
#relevant interactions
m.brt.simple.interacciones$rank.list

#TREE COMPLEXITY
###################################################
#calibramos los modelos
m.brt.simple.complexity.1=gbm.step(data=presencia.pseudoausencia, gbm.x = names(variables), gbm.y = "presencia", tree.complexity=1)

m.brt.simple.complexity.10=gbm.step(data=presencia.pseudoausencia, gbm.x = names(variables), gbm.y = "presencia", tree.complexity=10)

#evaluación
m.brt.simple.complexity.1$cv.statistics$discrimination.mean
m.brt.simple.complexity.10$cv.statistics$discrimination.mean

#QUÉ VALOR ES MEJOR
#ANALISIS DE SENSIBILIDAD
##########################################
complexity.vector<-seq(1, 46, by=5) #el máximo es 50
auc.vector<-vector()

#iteramos sobre valores de regmult
for (complexity.valor in complexity.vector){
  
  #ajustamos modelo
  m.brt.temp=gbm.step(data=presencia.pseudoausencia, gbm.x = names(variables), gbm.y = "presencia", tree.complexity=complexity.valor)
  
  #evaluamos el modelo y guardamos el valor de evaluación en auc.vector
  auc.vector<-c(auc.vector, evaluate(
    p=presencia.ausencia.entrenamiento[presencia.ausencia.entrenamiento$presencia==1, c("x", "y")], 
    a=presencia.ausencia.entrenamiento[presencia.ausencia.entrenamiento$presencia==0, c("x", "y")], 
    model=m.brt.temp, 
    n.trees=m.brt.temp$gbm.call$best.trees,
    x=variables
  )@auc)
  
}#final de las iteraciones

#vemos el resultado
plot(complexity.vector, auc.vector, type="l", xlab="tree complexity", ylab="auc")
#CONCLUSIÓN: UNA complejidad MAYOR INCREMENTA LA CAPACIDAD PREDICTIVA DEL MODELO.


#CALIBRAMOS MODELOS CON CADA TIPO DE PRESENCIAS
##################################################

#background
m.brt.background=gbm.step(data=presencia.background, gbm.x = names(variables), gbm.y = "presencia", tree.complexity = 40, learning.rate = 0.01)

#weighted background
m.brt.backgroundw=gbm.step(data=presencia.background, gbm.x = names(variables), gbm.y = "presencia", tree.complexity = 40, learning.rate = 0.01, site.weights=WeightPresenceBackground(presencia.background[ , "presencia"]))

#pseudoabsencce
m.brt.pseudoausencia=gbm.step(data=presencia.pseudoausencia, gbm.x = names(variables), gbm.y = "presencia", tree.complexity = 40, learning.rate = 0.01)

#ausencia
m.brt.ausencia=gbm.step(data=presencia.ausencia, gbm.x = names(variables), gbm.y = "presencia", tree.complexity = 40, learning.rate = 0.01)

#curvas de respuesta
x11()
plotmo(m.brt.background, caption="background")
x11()
plotmo(m.brt.backgroundw, caption="background ponderado")
x11()
plotmo(m.brt.pseudoausencia, caption="pseudoausencia")
x11()
plotmo(m.brt.ausencia, caption="ausencia")

#mapas
m.brt.background.map<-predict(variables, m.brt.background, n.trees=m.brt.background$gbm.call$best.trees, type="response")

m.brt.backgroundw.map<-predict(variables, m.brt.backgroundw, n.trees=m.brt.backgroundw$gbm.call$best.trees, type="response")

m.brt.ausencia.map<-predict(variables, m.brt.ausencia, n.trees=m.brt.ausencia$gbm.call$best.trees, type="response")

m.brt.pseudoausencia.map<-predict(variables, m.brt.pseudoausencia, n.trees=m.brt.pseudoausencia$gbm.call$best.trees, type="response")

#nombres y guardamos en modelos
names(m.brt.background.map)<-"brt.background"
names(m.brt.backgroundw.map)<-"brt.background.ponderado"
names(m.brt.ausencia.map)<-"brt.ausencia"
names(m.brt.pseudoausencia.map)<-"brt.pseudoausencia"
modelos.mapas<-addLayer(modelos.mapas, m.brt.background.map, m.brt.backgroundw.map, m.brt.ausencia.map, m.brt.pseudoausencia.map)

#map
x11()
par(mfrow=c(2,2))
plot(m.brt.background.map, main="background")
plot(m.brt.backgroundw.map, main="backgroundw")
plot(m.brt.ausencia.map, main="ausencia")
plot(m.brt.pseudoausencia.map, main="pseudoausencia")

#guardamos los objetos
modelos$brt.background<-m.brt.background
modelos$brt.background.ponderado<-m.brt.backgroundw
modelos$brt.pseudoausencia<-m.brt.pseudoausencia
modelos$brt.ausencia<-m.brt.ausencia

#borramos objetos que no necesitamos
rm(m.brt.ausencia.map, m.brt.background.map, m.brt.backgroundw.map, m.brt.simple.complexity.1, m.brt.simple.interacciones, m.brt.temp, m.brt.ausencia, m.brt.background, m.brt.pseudoausencia, m.brt.simple, m.brt.simple.complexity.10, m.brt.simple.map, m.brt.backgroundw, m.brt.pseudoausencia.map, complexity.valor, complexity.vector, auc.vector)
gc()


#######################################################################################
#######################################################################################
#ENSAMBLADO DE MODELOS
#######################################################################################
#######################################################################################


#SIMILITUD ENTRE LOS MODELOS
########################################
#transforma los modelos en dataframe eliminando nulos
modelos.df<-na.omit(as.data.frame(modelos.mapas))

#calcula matriz de correlaciones
modelos.cor<-cor(modelos.df)

#transforma matriz de correlación en distancias
modelos.dis<-abs(as.dist(modelos.cor))

#dibuja el árbol de correlación
x11()
par(mar=c(2,2,2,12))
plot(as.dendrogram(hclust(1-modelos.dis)),horiz=T)

#borra objetos que no vamos a usar más
rm(modelos.df, modelos.cor, modelos.dis)
gc()



#ENSAMBLADO
###########
#ENSAMBLADO MEDIANTE PROMEDIO
ensamblado.media<-calc(modelos.mapas, mean)

#DESVIACIÓN ESTÁNDAR
ensamblado.desviacion<-calc(modelos.mapas, sd)

#PLOT
x11()
par(mfrow=c(1,2), mar=c(1,1,1,1), oma=c(1,1,1,1))
plot(ensamblado.media, main="Media")
plot(ensamblado.desviacion, main="Desviación", col=rev(heat.colors(100)))

#PODEMOS FUNDIR AMBOS MAPAS
WhiteningEnsemble(average=ensamblado.media, deviation=ensamblado.desviacion)

#AÑADIMOS EL ENSAMBLADO A modelos
names(ensamblado.media)<-"ensamblado"
modelos<-addLayer(modelos, ensamblado.media)

#BORRAMOS OBJETOS QUE YA NO VAMOS A USAR
rm(ensamblado.desviacion, ensamblado.media, modelos.df, modelos.cor)
gc()


##################################
##################################
#EVALUACION DE MODELOS
##################################
##################################

#extraemos los valores de los puntos de evaluación en el modelo
valores.puntos.evaluacion<-extract(modelos.mapas, presencia.ausencia.evaluacion[, c("x","y")])
#ESTA TABLA CONTIENE LOS VALORES DE LAS PRESENCIAS Y AUSENCIAS DE EVALUACIÓN SOBRE CADA UNO DE LOS MODELOS, LA USAREMOS MÁS TARDE PARA OTRO EJERCICIO, NO TE OLVIDES DEL NOMBRE.
str(valores.puntos.evaluacion)

#es una lista, pasamos a data.frame
valores.puntos.evaluacion<-data.frame(valores.puntos.evaluacion)
str(valores.puntos.evaluacion)

#lo unimos con la columna presencia de presencias.test
valores.puntos.evaluacion$presencia<-presencia.ausencia.evaluacion$presencia
str(valores.puntos.evaluacion)

#creamos una tabla en blanco para guardar los resultados
resultados.evaluacion<-data.frame(modelo=character(), auc=numeric(), stringsAsFactors=FALSE)


#EMPIEZA A EJECUTAR EL LOOP AQUÍ
############################################
#abrimos un pdf para guardar los gráficos
pdf("./resultados/evaluacion_modelos.pdf", width=15, height=8, pointsize=30)

#contador de filas
fila=0

#iteramos por cada uno de los modelos
for (modelo in names(modelos.mapas)){
  
  #separamos los valores de las presencias y las ausencias
  #fíjate como en cada iteración tomamos la columna 'modelo'
  valores.presencias<-valores.puntos.evaluacion[valores.puntos.evaluacion$presencia==1, modelo]
  valores.ausencias<-valores.puntos.evaluacion[valores.puntos.evaluacion$presencia==0, modelo]
  
  #evaluamos el modelo
  evaluacion<-evaluate(p=valores.presencias, a=valores.ausencias)
  
  #suma 1 a la fila de la tabla de resultados
  fila=fila+1
  
  #llenamos la tabla
  resultados.evaluacion[fila, "modelo"]<-modelo
  resultados.evaluacion[fila, "auc"]<-evaluacion@auc
  
  #dibujamos el gráfico
  #plots
  par(mfrow=c(1,3), mar=c(2,2,4,2), oma=c(3,3,5,3))
  density(evaluacion)
  boxplot(evaluacion, col=c("blue", "red"))
  plot(evaluacion, "ROC")
  mtext(modelo, outer=TRUE, cex=1.3)
  
}

dev.off()
#TERMINA DE EJECUTAR EL LOOP AQUÍ

#ORDENAMOS LA TABLA RESULTANTE
resultados.evaluacion<-resultados.evaluacion[order(resultados.evaluacion$auc, decreasing=TRUE), ]
resultados.evaluacion #mejor lo vemos en el panel

#borramos cosas que no necesitamos
rm(valores.presencias, valores.ausencias, evaluacion, modelo, fila)
gc()


###########################################################
#EVALUANDO MODELOS CON K-FOLD
###########################################################

#seleccionando los datos a usar (los guardamos en otro objeto)
presencia.kfold<-presencia.pseudoausencia

#generating 10 grupos
k<-5
grupos<-kfold(presencia.pseudoausencia, k)

#para guardar resultados
auc.kfold<-vector()
mapas.kfold<-brick()

#formula del modelo (cámbiala según el tipo de modelo que uses más abajo)
formula.modelo<-as.formula(paste("presencia ~ ", paste(names(variables), collapse="+")))

#for every grupo
for (grupo in 1:k) {
  
  #seleccionamos las presencias para entrenar el modelo
  presencias.entrenamiento<-presencia.kfold[grupos != grupo,]
  
  #seleccionamos las presencias para evaluar el modelo
  presencias.evaluacion<-presencia.kfold[grupos == grupo,]
  
  #ajustamos el modelo aquí (cualquier otro modelo vale)
  modelo.entrenado<-randomForest(formula.modelo, data=presencias.entrenamiento, ntree=500, mtry=3, nodesize=10)
  
  #predice a  un mapa y lo guarda en un brick
  mapas.kfold<-addLayer(mapas.kfold, predict(variables, modelo.entrenado))
  
  #calculamos AUC
  auc.kfold[grupo]<-evaluate(p=predict(modelo.entrenado, presencias.evaluacion[presencias.evaluacion$presencia==1, ]), a=predict(modelo.entrenado, presencias.evaluacion[presencias.evaluacion$presencia==0, ]))@auc
  
} #terminamos iteracions

#resumen de los resultados
auc.kfold.mean<-mean(auc.kfold)
auc.kfold.mean
auc.kfold.sd<-sd(auc.kfold)
auc.kfold.sd
#ensemble
mapas.kfold.mean<-calc(mapas.kfold, mean)
mapas.kfold.sd<-calc(mapas.kfold, sd)

#mapas
x11()
par(mfrow=c(1,2))
plot(mapas.kfold.mean, main="Media")
plot(mapas.kfold.sd, main="Desviación estándar")


###########################################################
#EVALUANDO MODELOS CON BOOTSTRAP
###########################################################

#calculando el número de datos de entrenamiento
presencia.kfold<-presencia.pseudoausencia
presencia.kfold.nrow=nrow(presencia.kfold)
porcentaje.datos.evaluacion<-40
filas.datos.evaluacion<-round((porcentaje.datos.evaluacion*presencia.kfold.nrow)/100, 0)

#number of iteraciones
iteraciones=10

#objetos para guardar auc y mapas
auc.bootstrap<-vector()
mapas.bootstrap<-brick()

#iteramos por los grupos
for (iteracion in 1:iteraciones){
  
  print(paste("Iteracion:", iteracion))
  
  #muestreo aleatorio de los datos de presencia
  sampling<-sample(x=presencia.kfold.nrow, size=filas.datos.evaluacion, replace=TRUE)
  
  #seleccionamos los datos de entrenamiento
  presencias.entrenamiento<-presencia.kfold[-sampling, ]
  
  #seleccionamos datos de evaluación
  presencias.evaluacion<-presencia.kfold[sampling, ]
  
  #ajustamos el modelo (cualquier modelo puede ir aquí)
  modelo.entrenado<-randomForest(formula.modelo, data=presencias.entrenamiento, ntree=500, mtry=3, nodesize=10)
  
  #predecimos a mapa y lo guardamos en el brick
  mapas.bootstrap<-addLayer(mapas.bootstrap, predict(variables, modelo.entrenado))
  
  #calculamos auc
  auc.bootstrap[iteracion]<-evaluate(p=predict(modelo.entrenado, presencias.evaluacion[presencias.evaluacion$presencia==1, ]), a=predict(modelo.entrenado, presencias.evaluacion[presencias.evaluacion$presencia==0, ]))@auc
  
} #final de las iteraciones

#viendo resultados
auc.bootstrap.mean<-mean(auc.bootstrap)
auc.bootstrap.mean
auc.bootstrap.sd<-sd(auc.bootstrap)
auc.bootstrap.sd
#ensemble
mapas.bootstrap.mean<-calc(mapas.bootstrap, mean)
mapas.bootstrap.sd<-calc(mapas.bootstrap, sd)

#plotting mapas
par(mfrow=c(1,2))
plot(mapas.bootstrap.mean, main="Mean")
plot(mapas.bootstrap.sd, main="Standard deviation")

#borramos cosas que no necesitamos
rm(auc.bootstrap, auc.bootstrap.mean, auc.bootstrap.sd, auc.kfold, auc.kfold.mean, auc.kfold.sd, filas.datos.evaluacion, formula.modelo, grupo, grupos, iteracion, iteraciones, k, mapas.bootstrap, mapas.bootstrap.mean, mapas.bootstrap.sd, mapas.kfold, mapas.kfold.mean, mapas.kfold.sd, modelo.entrenado, porcentaje.datos.evaluacion, presencia.kfold, presencia.kfold.nrow, sampling)
gc()

####################################################################################
####################################################################################
#APLICANDO THRESHOLDS
####################################################################################
####################################################################################

#############################################
#SELECCIÓN DE THRESHOLDS MEDIANTE PERCENTILES
#(PORCENTAJES DE OMISIÓN)
#############################################
#PARA CALCULAR THRESHOLDS NECESITAMOS LOS VALORES DE LOS PUNTOS DE PRESENCIA Y AUSENCIA DE EVALUACIÓN SOBRE LOS MODELOS, YA LOS TENEMOS EN valores.puntos.evaluacion
str(valores.puntos.evaluacion)

#VAMOS A TRABAJAR CON SOLO UNO DE LOS MODELOS
modelo.elegido<-"brt.ausencia"
threshold.data<-valores.puntos.evaluacion[ , c(modelo.elegido, "presencia")]
threshold.data

#ELIGE UN PERCENTIL (si eliges 0.1, seleccionarás un threshold que contenga el 90% de las presencias)
percentil=0.1

#CALCULAMOS EL THRESHOLD
umbral1<-quantile(threshold.data[threshold.data$presencia==1, modelo.elegido], probs=percentil, na.rm=TRUE)
umbral1

#DIBUJAMOS EL MAPA (el modelo original está en el brick 'modelos')
x11()
plot(modelos.mapas[[modelo.elegido]] > umbral1)

#NÚMERO DE CELDAS DENTRO DEL UMBRAL
length(Which(modelos.mapas[[modelo.elegido]] > umbral1, cells=TRUE))



##########################################################
#SELECCIÓN DE THRESHOLD MEDIANTE TÉCNICAS DE OPTIMIZACIÓN
##########################################################

#separamos las presencias y las ausencias de valores.presencias.rf.pseudoausencia
threshold.data.presencia<-threshold.data[threshold.data$presencia == 1 , modelo.elegido]
threshold.data.ausencia<-threshold.data[threshold.data$presencia == 0 , modelo.elegido]

#LA FUNCIÓN EVALUATE PERMITE CALCULAR THRESHOLDS
#evaluamos
evaluacion.threshold<-evaluate(p=threshold.data.presencia, a=threshold.data.ausencia)
evaluacion.threshold
#fíjate en max TPR+TNR at : umbral
#ese valor es un threshold que maximiza la suma de clasificaciones correctas de presencias y ausencias
#TPR: TRUE POSITIVE RATE
#TNR: TRUE NEGATIVE RATE

#¿COMO SE CALCULA A PARTIR DE LO QUE HAY EN EL OBJETO evaluacion.rf.pseudoausencia?
umbral2<-evaluacion.threshold@t[which.max(evaluacion.threshold@TPR + evaluacion.threshold@TNR)]
umbral2

x11()
plot(modelos.mapas[[modelo.elegido]] > umbral2)


#TAMBIÉN PERMITE CALCULAR THRESHOLDS MAXIMIZANDO OTRAS MEDIDAS DE EVALUACIÓN
#THRESHOLD MAXIMIZANDO KAPPA
umbral3<-evaluacion.threshold@t[which.max(evaluacion.threshold@kappa)]
umbral3

#THRESHOLD MAXIMIZANDO CORRECT CLASSIFICATION RATE
umbral4<-evaluacion.threshold@t[which.max(evaluacion.threshold@CCR)]
umbral4

#EN TODOS ESTOS CASOS DA EL MISMO RESULTADO, PERO NO SIEMPRE ES ASÍ

#PLOT
dev.off()
x11()
par(mfrow=c(2,2))
plot(modelos.mapas[[modelo.elegido]] > umbral1, main="Solo presencia")
plot(modelos.mapas[[modelo.elegido]] > umbral2, main="Corte presencias-ausencias")
plot(modelos.mapas[[modelo.elegido]] > umbral3, main="max TPR+TNR")
plot(modelos.mapas[[modelo.elegido]] > umbral4, main="max CCR")

#borramos objetos que no vamos a utilizar
rm(threshold.data, threshold.data.presencia, umbral1, umbral2, umbral3, umbral4, threshold.data.ausencia, evaluacion.threshold, modelo.elegido, percentil, valores.puntos.evaluacion)
gc()



######################################################################################
######################################################################################
#PROYECCIÓN EN EL ESPACIO
######################################################################################
######################################################################################

#CAMBIAMOS EL NOMBRE A LAS VARIABLES DE EUROPA
variables.eu<-variables

#IMPORTAMOS LAS VARIABLES DE NORTEAMÉRICA
######################################################
#descomprimimos las variables
unzip("./data/3_variables_norteamerica.zip", exdir="./resultados/variables_norteamerica", junkpaths=TRUE)

#LISTADO DE VARIABLES
lista.variables <- list.files(path="./resultados/variables_norteamerica",pattern='*.asc', full.names=TRUE)
#brick
variables.na <- brick(stack(lista.variables))
#comprobamos los nombres de las variables
names(variables.na)
#son muchas variables, nos quedamos con las las mismas que hay en variables.eu
variables.na<-variables.na[[names(variables.eu)]]
#plot
x11()
plot(variables.na)


#ANÁLISIS MESS (multivariate environmental similarity surfaces, Elith et al. 2010)
#UNA PREGUNTA IMPORTANTE: EXISTEN EN NA LAS CONDICIONES REQUERIDAS POR LA ESPECIE EN EU?
######################################################
#necesita registros de presencia con los valores de las variables, y un brick con las variables de la región de destino
mess.na<-mess(x=variables.na, v=presencia[, names(variables.eu)], full=TRUE)
names(mess.na)<-c(names(variables.na), "mess")
mess.na<-mask(mess.na, mask=variables.na[[1]])
dev.off()
x11()
plot(mess.na)

#INTERPRETACIÓN: VALORES MAYORES QUE 20 Y MENORES QUE -20 INDICAN COMBINACIONES DE CONDICIONES AMBIENTALES NO REPRESENTADAS EN LAS PRESENCIAS DE ENTRENAMIENTO.
#tomamos el mapa del mess global
mess.mapa<-mess.na[["mess"]]
breaks<-c(minValue(mess.mapa), -20, 20, maxValue(mess.mapa))
colores<-c("red", "blue", "red")
plot(mess.mapa, breaks=breaks, col=colores)
#CUALQUIER MODELO CALIBRADO CON TODAS LAS VARIABLES ESTARÁ EXTRAPOLANDO EN LAS ÁREAS ROJAS.


#¿DONDE ESTÁ EL DESVÍO MÁXIMO DE CADA VARIABLE?
dev.off()
mess.na.max<-which.max(abs(mess.na)) #no funciona según la versión de R y librería Raster
mess.na.max<-mask(mess.na.max, mask=mess.na[[1]])
plot(mess.na.max)
#le ponemos una paleta de color más intuitiva
#creamos una paleta con 8 colores
paleta.color<-brewer.pal(n=8, name="Set1")
#plot con leyenda
x11()
plot(mess.na.max, col=paleta.color, legend=FALSE)
legend("bottomleft", c(names(variables.na)), pch=15, col=paleta.color, cex=1, border="")


#borra objetos que no vamos a necesitar
rm(lista.variables, variables)
gc()



#PROYECCIÓN DE MODELOS
#######################
#YA TENEMOS TODOS NUESTROS MODELOS GUARDADOS EN modelos
names(modelos)

#PARA PROYECTAR UN MODELO CONCRETO SOBRE NORTE AMÉRICA, SOLO HAY QUE HACER...
proyeccion.brt<-predict(variables.na, modelos$brt.ausencia, type="response", n.trees=modelos$brt.ausencia$gbm.call$best.trees)

x11()
plot(proyeccion.brt, main="brt")


#COMO SABEMOS SI LA PROYECCIÓN ES BUENA O NO
#importamos presencias limpias y ausencias generadas de la misma forma que hicimos para Europa
presencia.na<-read.table("./data/presencia_na.csv", header=T, sep=';', stringsAsFactors=FALSE)
str(presencia.na)

#LAS DIBUJAMOS
dev.off()
x11()
plot(proyeccion.brt, main="brt")
points(presencia.na[presencia.na$presencia==1, "x"], presencia.na[presencia.na$presencia==1, "y"], col="blue")
points(presencia.na[presencia.na$presencia==0, "x"], presencia.na[presencia.na$presencia==0, "y"], col="red")

#EVALUAMOS LA PROYECCION
#extraemos valores de las presencias y ausencias para el modelo
valores.presencias<-data.frame(presencia.na, idoneidad=extract(proyeccion.brt, presencia.na[, c("x","y")]))

#evaluamos el modelo
evaluate(p=valores.presencias[valores.presencias$presencia==1, "idoneidad"], a=valores.presencias[valores.presencias$presencia==0, "idoneidad"])


##########################################################################
#AVANZADO: PROYECTAMOS Y EVALUAMOS TODOS LOS MODELOS
##########################################################################
#creamos brick para guardar los modelos
modelos.proyectados.mapas<-brick()

#tenemos una tabla para guardar los resultados, le añadimos una columna
resultados.evaluacion$auc.proyeccion<-NA

#iteramos por cada modelo en modelos
for(modelo in names(modelos)){
  
  #si el modelo es brt
  if(length(grep(pattern="brt", x=modelo))>0){
    proyeccion<-predict(variables.na, modelos[[modelo]], type="response", n.trees=modelos[[modelo]]$gbm.call$best.trees)
  } 
  
  #si el modelo es tree quitamos type="response"
  if(length(grep(pattern="tree", x=modelo))>0){
    proyeccion<-predict(variables.na, modelos[[modelo]])
  } 
  
  #si el modelo es tree quitamos type="response"
  if(length(grep(pattern="maxent", x=modelo))>0){
    proyeccion<-predict(variables.na, modelos[[modelo]], type="logistic")
  } 
  
  #other models without brt, tree, or maxent
  if(length(grep(pattern="brt", x=modelo))==0 & length(grep(pattern="tree", x=modelo))==0 & length(grep(pattern="maxent", x=modelo))==0){
    proyeccion<-predict(variables.na, modelos[[modelo]], type="response")
  }

  #nombramos la proyeccion y la guardamos
  names(proyeccion)<-modelo
  modelos.proyectados.mapas<-addLayer(modelos.proyectados.mapas, proyeccion)
  
  #extraemos valores de las presencias y ausencias para el modelo
  valores.presencias<-data.frame(presencia.na, idoneidad=extract(proyeccion, presencia.na[, c("x","y")]))
  
  #evaluamos el modelo y guardamos el resultado de la evaluacion
  resultados.evaluacion[resultados.evaluacion$modelo==modelo, "auc.proyeccion"] <- evaluate(p=valores.presencias[valores.presencias$presencia==1, "idoneidad"], a=valores.presencias[valores.presencias$presencia==0, "idoneidad"])@auc
  
}

#creamos columnas con el ranking the cada auc
resultados.evaluacion$auc.rank<-1:nrow(resultados.evaluacion)
resultados.evaluacion<-resultados.evaluacion[order(resultados.evaluacion$auc.proyeccion, decreasing=TRUE), ]
resultados.evaluacion$auc.proyeccion.rank<-1:nrow(resultados.evaluacion)
resultados.evaluacion$rank.suma<-resultados.evaluacion$auc.proyeccion.rank+resultados.evaluacion$auc.rank
resultados.evaluacion<-resultados.evaluacion[order(resultados.evaluacion$rank.suma, decreasing=FALSE), ]

#vemos la tabla
#no comparamos auc y auc.proyeccion, pero comparamos sus rankings
#el modelo con menor rank.suma es el que mejores valores de auc tiene para ambos modelos
resultados.evaluacion

#ploteamos algunas de las mejores proyecciones
x11()
par(mfrow=c(2,2))
plot(modelos.proyectados.mapas$maxent, main="maxent")
plot(modelos.proyectados.mapas$brt.pseudoausencia, main="brt.pseudoausencia")
plot(modelos.proyectados.mapas$brt.background.ponderado, main="brt.background.ponderado")
plot(modelos.proyectados.mapas$rf.pseudoausencia, main="rf.pseudoausencia")

#borramos cosas que no necesitamos
rm(proyeccion.brt, valores.presencias, proyeccion, mess.na, breaks, modelo, mess.mapa, mess.na.max, colores, paleta.color)
gc()

##################################################################################
#COMPARANDO LOS NICHOS
##################################################################################
# install.packages("ecospat")
library(ecospat)

#presencias y valores de las variables en Norte América
valores.na<-data.frame(extract(variables.na, presencia.na[, c("x","y")]))
presencia.na<-data.frame(y=presencia.na$y, x=presencia.na$x, valores.na, presencia=presencia.na$presencia)

#presencias y valores de variables en Europa
presencia.eu<-presencia.pseudoausencia

#vemos si tienen los mismos nombres de columna
names(presencia.na)==names(presencia.eu)

#PCA con todos los datos
pca.eu.na <- dudi.pca(rbind(presencia.eu,presencia.na)[,names(variables.eu)],scannf=F,nf=2)

#PCA scores area completa
scores.eu.na <- pca.eu.na$li

#PCA scores presencias EU
scores.eu.presencias <- suprow(pca.eu.na, presencia.eu[which(presencia.eu$presencia==1), names(variables.eu)])$li

# PCA scores para el area EU
scores.eu.area <- suprow(pca.eu.na, presencia.eu[,names(variables.eu)])$li

#PCA scores presencias NA
scores.na.presencias <- suprow(pca.eu.na, presencia.na[which(presencia.na$presencia==1), names(variables.eu)])$li

# PCA scores para el area NA
scores.na.area <- suprow(pca.eu.na, presencia.na[,names(variables.eu)])$li

#PCA scores de cada sitio a grid
scores.eu.grid <- ecospat.grid.clim.dyn(glob=scores.eu.na, glob1=scores.eu.area, sp=scores.eu.presencias, R=100, th.sp=0)
scores.na.grid <- ecospat.grid.clim.dyn(glob=scores.eu.na, glob1=scores.na.area, sp=scores.na.presencias, R=100, th.sp=0)

#NICHE OVERLAP Schoener's D
D.overlap <- ecospat.niche.overlap(scores.eu.grid, scores.na.grid, cor=T)$D
D.overlap


#NICHE EQUIVALENCY TEST
#Es el solapamiento entre los dos nichos mayor que el de dos nichos aleatorios?
#Se recomienda al menos rep=1000
eq.test <- ecospat.niche.equivalency.test(scores.eu.grid, scores.na.grid, rep=100, alternative = "greater")
eq.test


#Dibuja el plot de solapamiento de nicho
dev.off()
x11()
ecospat.plot.niche.dyn(scores.eu.grid, scores.na.grid, quant=0.25, interest=2, title= "Niche Overlap", name.axis1="PC1", name.axis2="PC2")


###################################################################################
###################################################################################
#PROYECCIÓN EN EL TIEMPO
###################################################################################
###################################################################################

#NOS QUEDAMOS SOLO CON LOS NOMBRES DE LAS VARIABLES CLIMÁTICAS QUE HEMOS UTILIZADO
#vemos los nombres
lista.variables<-names(variables.eu)
#tomamos solo los que necesitamos
lista.variables<-c(lista.variables[[1]], lista.variables[[2]], lista.variables[[6]])
#vemos que todo salió bien
lista.variables
#creamos un brick nuevo
variables.presente<-variables.eu[[lista.variables]]


################################################
#IMPORTAMOS LAS VARIABLES DE PROYECCIÒN TEMPORAL
################################################
#descomprimimos las variables
unzip("./data/4_proyeccion_temporal.zip", exdir="./resultados/variables_proyeccion_tiempo")


#VARIABLES PALEO 21 Kyr
########################
lista.variables.paleo.21k <- list.files(path="./resultados/variables_proyeccion_tiempo/variables_paleo_21k",pattern='*.asc', full.names=TRUE)
#brick
variables.paleo.21k <- brick(stack(lista.variables.paleo.21k))
#nos quedamos solo con las necesarias
variables.paleo.21k<-variables.paleo.21k[[lista.variables]]


#VARIABLES FUTURO
#################
lista.variables.futuro <- list.files(path="./resultados/variables_proyeccion_tiempo/variables_futuro_ccsm_A1B",pattern='*.asc', full.names=TRUE)
#brick
variables.futuro <- brick(stack(lista.variables.futuro))
#nos quedamos solo con las necesarias
variables.futuro<-variables.futuro[[lista.variables]]

#ENTRENAMIENTO DEL MODELO
#############################

#PREPARAMOS LAS FÓRMULAS (SOLO LAS TRES VARIABLES CLIMÁTICAS)
formula.regresion.poly<-as.formula(paste("presencia ~ poly(", paste(names(variables.presente), collapse=", 2) + poly("), ", 2)", collapse=""))
formula.regresion.poly

#CALCULAMOS PESOS PARA LOS BACKGROUND PONDERADOS DE GLM Y GAM
pesos<-WeightPresenceBackground(presence.column=presencia.background[ , "presencia"])

#CALIBRAMOS EL MODELO
m.glm<-glm(formula.regresion.poly, family=quasibinomial(link=logit), data=presencia.background, weights=pesos)

#VEMOS CARACTERÍSTICAS DEL MODELO
summary(m.glm)
Dsquared(m.glm)
plotmo(m.glm, all2=TRUE, level=0.68)

#PROYECTAMOS SOBRE LAS VARIABLES DEL PRESENTE
m.glm.map.presente<-predict(variables.presente, m.glm, type="response")

#PROYECTAMOS SOBRE LAS VARIABLES DEL 21k
m.glm.map.paleo.21k<-predict(variables.paleo.21k, m.glm, type="response")

#PROYECTAMOS SOBRE LAS VARIABLES DEL FUTURO CSIRO
m.glm.map.futuro<-predict(variables.futuro, m.glm, type="response")


#PLOTEAMOS LOS MODELOS
######################
x11()
par(mfrow=c(1,3), mar=c(1,1,1,1), oma=c(3,3,4,3))
plot(m.glm.map.paleo.21k, main="GLM - 21 kyr BP")
plot(m.glm.map.presente, main="GLM - Present")
plot(m.glm.map.futuro, main="GLM - 2060")

